#!/usr/bin/perl
###################################################################
###      File: filter-grammar.perl                              ###
###    Author: Helmut Schmid                                    ###
###   Purpose:                                                  ###
###   Created: Fri Jan 31 10:25:22 2003                         ###
###  Modified: Fri Nov 25 11:22:15 2005 (schmid)                ###
### Copyright: Institut fuer maschinelle Sprachverarbeitung     ###
###               Universitaet Stuttgart                        ###
###################################################################

use Getopt::Std;
getopts('h');

$maxsymlen = 200;

if (defined $opt_h) {
  die "
Usage: filter-grammar.perl <grammar>

";
}

print STDERR "reading rules... ";
while (<>) {
  chomp;
  last if $_ eq '';
  next if /\|/;  # ignore rules ambiguous POS tags
  my($freq,@tags) = split;
  my $rule;
  my $delete=0;
  foreach $tag (@tags) {
    if (length($tag) > $maxsymlen) {
      print STDERR "Warning: Rule deleted because symbol length exceeds $maxsymlen:\n$_\n";
      $delete = 1;
      last;
    }
    $known_tag{$tag} = 1;
  }
  $count{join(' ',@tags)} += $freq unless $delete;
}


print STDERR "finished\nreading lexicon... ";
while (<>) {
  chomp;
  my($freq,$tag,$word) = split;
  next unless exists $known_tag{$tag};
  next if $tag =~ /\|/;
  $tcount{"$word\t$tag"} += $freq;
}

print STDERR "finished\nwriting grammar... ";
open(FILE, ">grammar") or die "Error: unable to open file \"grammar\"!\n";
foreach $r (sort {$count{$b} <=> $count{$a}} keys %count) {
  print FILE "$count{$r}\t$r\n";
}
close FILE;

print STDERR "finished\nwriting lexicon... ";

open(FILE, ">lexicon") or die "Error: unable to open file \"lexicon\"!\n";
foreach $p (sort keys %tcount) {
  my ($w,$t) = split('\t',$p);
  if ($w ne $pw) {
    print FILE  "\n" if defined $pw;
    print FILE $w;
    $pw = $w;
  }
  print FILE "\t$t $tcount{$p}";
}
print FILE "\n";
close FILE;

print STDERR "finished\n";
