#!/usr/bin/perl
###################################################################
###      File: binarize-rules.perl                              ###
###    Author: Helmut Schmid                                    ###
###   Purpose:                                                  ###
###   Created: Thu Jul 21 12:01:31 2005                         ###
###  Modified: Mon Feb 27 14:04:15 2006 (schmid)                ###
### Copyright: Institut fuer maschinelle Sprachverarbeitung     ###
###               Universitaet Stuttgart                        ###
###################################################################

use Getopt::Std;
getopts('t:r:');

$Threshold1 = (defined $opt_t)? $opt_t: 10;
$Threshold2 = (defined $opt_r)? $opt_r: $Threshold1;

open(FILE,"head-rules") or die "Error: unable to open file \"head-rules\"!\n";
for( $i=0; <FILE>; $i++) {
  # normalize the input
  s/\s*\t\s*/\t/g;
  s/  +/ /g;
  s/^\s+//g;
  s/\s+$//g;
  # split the input
  my($c,$d,@s)=split(/\t/);
  die "Error: in line $i of file \"head-rules\":\n$_" 
    unless $d eq "right" or $d eq "left";
  $dir{$c} = $d;
  $set{$c} = \@s;
}
close FILE;

while (<>) {
  ($freq, @node) = split;

  # no markovization of binary rules and rules with gap features 
  if ($#node <= 2 || / .*/ || $freq >= $Threshold1) {
    $count{join(' ',@node)} += $freq;
  }
  else {
    # generate and count the trigram rules
    $mother = $node[0];
    $head = $node[head_number()];
    $m = $mother;
    for( $i=$#node; $i>2; $i-- ) {
      $d = "$mother $head $node[$i-1] $node[$i]";
      $r  = "$m\t$d\t$node[$i]";
      $bcount{$r} += $freq;
      $ccount{$d} += $freq;
      $m = $d;
    }
    $r = "$m\t$node[1]\t$node[2]";
    $bcount{$r} += $freq;
  }
}

# replace trigram non-terminals with bigram non-terminals
# if the trigram frequency is below the threshold
foreach $r (sort keys %bcount) {
  @F = split(/\t/,$r);
  for( $i=0; $i<=$#F; $i++ ) {
    if ($F[$i] =~ /^(\S*) (\S+) (\S+) (\S+)$/) {
      if ($ccount{$F[$i]} < $Threshold2) {
	$F[$i] = "<$1\[$2\]$3>";
      }
      else {
	$F[$i] = "<$1\[$2\]$3|$4>";
      }
    }
  }
  $count{join(' ',@F)} += $bcount{$r};
}

# print the new rules
foreach $r (sort {$count{$b} <=> $count{$a}} keys %count) {
  print "$count{$r} $r\n";
}


sub head_number {
  my(@cat,$mother,$i,$k,@set);
  # remove any features
  for( $i=0; $i<=$#node; $i++ ) {
    $cat[$i] = $node[$i];
    $cat[$i] =~ s/[\/\-\+\_\%\=].*//;
    $cat[$i] =~ s/^V[^P]/VB/;
  }

  # find the rule
  $mother=$cat[0];
  @set = @{$set{$mother}};
  if ($dir{$mother} eq "left") {
    for( $i=0; $i<=$#set; $i++ ) {
      my %mcat = map { $_ => 1 } split(/ /,$set[$i]);
      for( $k=1; $k<=$#cat; $k++ ) {
	return $k if exists $mcat{$cat[$k]};
      }
    }
    for( $k=1; $k<=$#cat; $k++ ) {
      return $k if $cat[$k] =~ /^[A-Z]/;
    }
    printf STDERR "Warning: unable to determine the head of the rule: ",
      join(' ',@node),"\n";
    return 1;
  }
  else {
    for( $i=0; $i<=$#set; $i++ ) {
      my %mcat = map { $_ => 1 } split(/ /,$set[$i]);
      for( $k=$#cat; $k>=1; $k-- ) {
	return $k if exists $mcat{$cat[$k]};
      }
    }
    for( $k=$#cat; $k>=1; $k-- ) {
      return $k if $cat[$k] =~ /^[A-Z]/;
    }
    printf STDERR "Warning: unable to determine the head of the rule: ",
      join(' ',@node),"\n";
    return $#node;
  }
}
