#!/bin/sh

# This script generates a treebank grammar using tmod,
# parses the treebank with this grammar,
# and computes precision and recall figures

./make-grammar.sh

echo "Parsing ..."
gzip -cd DATA/test.txt.gz | cmd/extract-words.perl  |
bitpar -S 4 -ts '()' -s TOP -v grammar lexicon -u open-class-tags -w wordclass.txt -n grammar.nf > test.parses

echo "Evaluation ..."
./tmod -u test.parses | cmd/normalise-parse.perl > t.tst
gzip -cd DATA/test.txt.gz | cmd/normalise-penn.perl > t.gold
evalb -p DATA/evalb.prm t.gold t.tst > result

echo "grammar size:" `cat grammar | wc -l` >> result
echo "parse failures:" `grep -c 'No parse for:' test.parses` >> result

gzip -cd DATA/test.txt.gz | ./tmod -e > gold
sed 's/^No parse for: .*/(NP(NN car))/' test.parses | ./tmod -u | ./tmod -e > test
cmd/link-traces.perl gold > gold1
cmd/link-traces.perl test > test1
cmd/compare-traces.perl gold1 test1 | sort -n > comp
cmd/compute-score.perl comp | awk '$0=="Antecedent Recovery"{exit}{print}' >> result

sed 's/SBARXS/S/g' gold | cmd/link-traces.perl > gold1
sed 's/SBARXS/S/g' test | cmd/link-traces.perl > test1
cmd/compare-traces.perl gold1 test1 | sort -n > comp
cmd/compute-score.perl comp  | awk '$0=="Antecedent Recovery"{p=1}p{print}' >> result

/bin/rm t.gold t.tst gold gold1 test test1 comp

