
/*MA****************************************************************/
/*                                                                 */
/*     File: rules.C                                               */
/*   Author: Helmut Schmid                                         */
/*  Purpose:                                                       */
/*  Created: Mon Jan 27 10:26:12 2003                              */
/* Modified: Thu Sep 15 17:33:29 2005 (schmid)                     */
/*                                                                 */
/*ME****************************************************************/


#include "tree.h"
#include "frequency-table.h"

FrequencyTable LF, GF;


/*FA****************************************************************/
/*                                                                 */
/*  add_string                                                     */
/*                                                                 */
/*FE****************************************************************/

static char *add_string( char *p, char *s )

{
  while ((*(p++) = *(s++))) ;
  return p-1;
}


/*FA****************************************************************/
/*                                                                 */
/*  Tree::add_rules                                                */
/*                                                                 */
/*FE****************************************************************/

void Tree::add_rules()

{
  if (daughters.size() > 0) {
    static char buffer[10000];
    char *p = buffer;

    if (daughters.size() == 1 && daughters[0]->daughters.size() == 0) {
      // lexical entry
      sprintf(buffer,"%s %s", label, daughters[0]->label);
      LF[buffer]++;
    }

    else {
      // grammar rule
      p = add_string(p, label);
      for( size_t i=0; i<daughters.size(); i++ ) {
	*(p++) = ' ';
	if (daughters[i] == head)
	  *(p++) = '^';
	p = add_string(p, daughters[i]->label);
      }
      GF[buffer]++;
    }

    for( size_t i=0; i<daughters.size(); i++ )
      daughters[i]->add_rules();
  }
}


/*FA****************************************************************/
/*                                                                 */
/*  print_grammar                                                  */
/*                                                                 */
/*FE****************************************************************/

void print_grammar()

{
  for( FrequencyTable::iterator it=GF.begin(); it!=GF.end(); it++ )
    fprintf(outfile,"%ld %s\n", it->second, it->first);

  if (lexfile == outfile)
    fputc('\n', outfile);

  for( FrequencyTable::iterator it=LF.begin(); it!=LF.end(); it++ )
    fprintf(lexfile,"%ld %s\n", it->second, it->first);
}
