
/*MA****************************************************************/
/*                                                                 */
/*     File: tree.h                                                */
/*   Author: Helmut Schmid                                         */
/*  Purpose:                                                       */
/*  Created: Tue Jan 21 13:27:46 2003                              */
/* Modified: Fri Nov 13 08:31:47 2009 (schmid)                     */
/*                                                                 */
/*ME****************************************************************/

#include <stdio.h>
#include <vector>
using std::vector;

#include "symbol-map.h"

extern bool Verbose;
extern bool PrintRules;
extern bool WithHeads;
extern bool PrettyPrint;
extern bool PrintTraces;
extern bool UndoMarkov;
extern bool EmptyCategories;
extern bool ConvertTraces;
extern char *MapFileName;
extern char *PrepFileName;
extern int  Trace_Number;

extern FILE *govfile, *outfile, *lexfile;

class Tree;

typedef vector<Tree*> Trees;

void read_mapping( char*, SymbolMap& );

extern SymbolMap CatMap;
extern SymbolMap PrepMap;


/*****************  class Tree  ************************************/

class Tree {

private:
  void assign_heads();
  void print_subtree(int);
  int print_ec2(int);

  void change_label( const char* );
  bool is_trace();
  bool is_first_daughter();
  bool is_last_daughter();
  bool is_preposition();
  Tree *first_daughter() { return daughters[0]; };
  Tree *last_daughter() { return daughters[daughters.size()-1]; };
  Tree *predecessor();
  Tree *successor();
  void strip_features();
  const char *base_cat();
  char *category();
  char *category2();
  bool matching_suffix( const char* );
  bool matching_base_cat( const char* );
  bool matching_category( const char* );
  bool noun_POS_tag();
  bool verb_POS_tag();
  bool has_trace_feature( char, const char* );
  char matching_trace( const char* );
  void append_slash( char*, char, bool );
  char append_slashes( char*, const char*, bool& );

  void correct();
  bool matching_prefix( const char *prefix );
  void add_markup( const char *s, char d );
  void insert_feature( const char *s );
  void insert_uniq_feature( const char *s );
  void VP_error();
  void add_genitive_markers();
  void add_TMP_markers();
  void add_MNR_markers();
  void add_NPqp_markers();
  void add_PP_features();
  void add_ADJ_features();
  void add_DT_features();
  void add_ADV_features();
  void add_NP_features();
  void add_V_features();
  void add_CD_features();
  void add_VP_features();
  void add_subcat_feature();
  void add_S_features();
  void add_SBAR_features();
  void add_CC_features();
  void add_UCP_features();
  void add_LST_features();
  void add_CONJP_features();
  char *add_punctuation_feature();
  void add_oc_features();
  bool add_domV_feature();
  void add_gapped_feature();
  void add_slashes();
  void mark_as_trace();
  bool collapse_traces();
  bool delete_traces();
  void delete_indices();
  void add_rules();
  void print_govs();
  const char *cut_off_indices();
  void undo_markov();
  void print_ec();

  void add_trace_feature( char c );
  char *slash_cat( char *c );
  char *find_trace( char*, char, bool& );
  bool delete_slashes_down( char *sc, char d );
  bool delete_slashes_up( char *sc, char d );
  bool convert( bool& );

public:
  char  *label;
  Trees daughters;
  Tree  *mother;
  size_t catnum;
  Tree  *head_daughter;
  Tree  *head;

  Tree( char *l ) { label = l; mother = head_daughter = head = NULL; };

  Tree( char *l, char *l2 ) {
    label = l;
    daughters.push_back(new Tree(l2));
    mother = head_daughter = head = NULL; 
  };

  Tree( char *l, Trees *d ) { 
    label = l;
    daughters = *d;
    for( size_t i=0; i<daughters.size(); i++ )
      daughters[i]->mother = this;
    mother = head_daughter = head = NULL; 
  };

  ~Tree() {
    free(label);
    for( size_t i=0; i<daughters.size(); i++ )
      delete daughters[i];
  }

  void process();
  void print();
};

void print_grammar();


char *last_char( char *s );
bool index_start( const char *s );
const char *last_char( const char *s );

void read_head_rules( char *filename );
